#include <stdio.h>
#include <math.h>

/* ---------------------------------------------------------------- */

// dfinition d'un type "gnrique" de rels
// "valeurs" possibles : float, double
#define TYPENUM double

// dfinition d'un type de fonction mathmatique
// (x,y) -> f(x,y)
typedef TYPENUM func_t(TYPENUM x,TYPENUM y);

/* ---------------------------------------------------------------- */

/* mthode de Runge-Kutta d'ordre 4 */

// on cherche numriquement la solution de 
// df/dx = g(x,f(x)) et f(x0)=y0
// est le nombre de points intermdiaires
TYPENUM rk4(func_t *g, TYPENUM x, TYPENUM x0, TYPENUM y0, long n)
{
  long i; // indice de boucle (entier long)

  TYPENUM h = (x-x0)/n;    // pas
  TYPENUM xi, yi;          // point intermdiaire
  TYPENUM k1, k2, k3, k4;  // valeurs internes

  // fixe les conditions initiales
  xi = x0;
  yi = y0;

  // boucle pour n points intermdiaires
  for (i=0; i<n; i++)
    {
      k1 = h*g(xi,yi);
      k2 = h*g(xi+h/2.0,yi+k1/2.0);
      k3 = h*g(xi+h/2.0,yi+k2/2.0);
      k4 = h*g(xi+h,yi+k3);
      xi += h;
      yi += (1.0/6.0)*(k1+2.0*k2+2.0*k3+k4);
    }

  return yi;
}

/* ---------------------------------------------------------------- */

/* fonction tudie */
TYPENUM f(TYPENUM x, TYPENUM y)
{
  return 10.0*(y-x*x);
}

/* ---------------------------------------------------------------- */
int main ()
{
  TYPENUM x0 = 0.0;
  TYPENUM y0 = 0.02;

  long n = 100000;

  printf("%g\n",rk4((func_t*)f,1.0,x0,y0,n));
  printf("%g\n",rk4((func_t*)f,2.0,x0,y0,n));
  printf("%g\n",rk4((func_t*)f,3.0,x0,y0,n));
  printf("%g\n",rk4((func_t*)f,4.0,x0,y0,n));
  printf("%g\n",rk4((func_t*)f,5.0,x0,y0,n));
  printf("%g\n",rk4((func_t*)f,6.0,x0,y0,n));

  return 0;
}
